#include "FGDependency.h"

#include <sstream>

using namespace std;

namespace egstra {
  void fg_dependency::phi(const sentence* x, list<string> F[],
			  const int word_limit,
			  const bool use_pos) {
    const int N = x->size();
    ostringstream os;
    for(int h = 0; h < N; ++h) {
      for(int m = 0; m < N; ++m) {
	const int dep_idx = h + N*m;
	list<string>& f = F[dep_idx];

	/* indicator for the direction of the dependency */
	const int modBefore = (m < h ? 1 : 0);

	/* gather information about head and modifier */
	int hword, hctag;
	if(h == m) { /* head is root */
	  hword = hctag = -1;
	} else {
	  const token& ht = x->get_token(h);
	  hword = (ht.word() < word_limit ? ht.word() : -2);
	  hctag = ht.coarse_tag();
	}
	const token& mt = x->get_token(m);
	const int mword = (mt.word() < word_limit ? mt.word() : -2);
	const int mctag = mt.coarse_tag();

	/* add features based on head/modifier word/tag */

	if(use_pos) {
	  /* child word + child ctag + head word + head ctag */
	  os.str("");
	  os << "cw+ct+hw+ht:" << modBefore << "+"
	     << mword << "+" << mctag << "+"
	     << hword << "+" << hctag;
	  f.push_back(os.str());

	  /* child word + head word + head ctag */
	  os.str("");
	  os << "cw+hw+ht:" << modBefore << "+"
	     << mword << "+"
	     << hword << "+" << hctag;
	  f.push_back(os.str());

	  /* child word + child ctag + head word */
	  os.str("");
	  os << "cw+ct+hw:" << modBefore << "+"
	     << mword << "+" << mctag << "+"
	     << hword;
	  f.push_back(os.str());

	  /* child ctag + head word + head ctag */
	  os.str("");
	  os << "ct+hw+ht:" << modBefore << "+"
	     << mctag << "+"
	     << hword << "+" << hctag;
	  f.push_back(os.str());

	  /* child word + child ctag + head ctag */
	  os.str("");
	  os << "cw+ct+ht:" << modBefore << "+"
	     << mword << "+" << mctag << "+"
	     << hctag;
	  f.push_back(os.str());

	  /* child ctag + head ctag */
	  os.str("");
	  os << "ct+ht:" << modBefore << "+"
	     << mctag << "+" << hctag;
	  f.push_back(os.str());
	}

	/* child word + head word */
	os.str("");
	os << "cw+hw:" << modBefore << "+"
	   << mword << "+" << hword;
	f.push_back(os.str());
      }
    }
  }
}
