#include "FGUtil.h"

#include "CharUtils.h"
#include "StringMap.h"
#include "CppAssert.h"
#include "Constants.h"

#include <fstream>
#include <string.h>
#include <assert.h>

using namespace std;

namespace egstra {
  int fg_util::_ntags = -1;
  bool* fg_util::_tag_is_verb = NULL;
  bool* fg_util::_tag_is_punc = NULL;
  bool* fg_util::_tag_is_coord = NULL;

  void fg_util::load_tag_map(const string& path) {
    if(_tag_is_verb != NULL) {
      delete [] _tag_is_verb;
      delete [] _tag_is_punc;
      delete [] _tag_is_coord;
    }

    vector<bool> isverb;
    vector<bool> ispunc;
    vector<bool> iscoord;

    std::ifstream is(path.c_str());
    RVASSERT(is, "Can not open file: " << path);
    char line[MAX_FEATDICT_LINE];

    while(is.getline(line, MAX_FEATDICT_LINE)){
      vector<string> tokens;
      simpleTokenize(line, tokens, " \t");
      RVASSERT(tokens.size() == 2,
	       "Invalid line \"" << line << "\" in file: " << path);

      const int idx = toInteger(tokens[0]);
      const char* const tag = tokens[1].c_str();
      if(idx >= (int)isverb.size())  { isverb.resize(idx + 1, false); }
      if(idx >= (int)ispunc.size())  { ispunc.resize(idx + 1, false); }
      if(idx >= (int)iscoord.size()) { iscoord.resize(idx + 1, false); }
      /* check for membership */
      if(tag[0] == 'v' || tag[0] == 'V') {
	isverb[idx] = true;
      } else if(strcmp(tag, "Punc") == 0 ||
		strcmp(tag, "$,") == 0 ||
		strcmp(tag, "$.") == 0 ||
		strcmp(tag, "PUNC") == 0 ||
		strcmp(tag, "punc") == 0 ||
		strcmp(tag, "F") == 0 ||
		strcmp(tag, "IK") == 0 ||
		strcmp(tag, "XP") == 0 ||
		strcmp(tag, ",") == 0 ||
		strcmp(tag, ";") == 0) {
	ispunc[idx] = true;
      } else if(strcmp(tag, "Conj") == 0 ||
		strcmp(tag, "KON") == 0 ||
		strcmp(tag, "conj") == 0 ||
		strcmp(tag, "Conjunction") == 0 ||
		strcmp(tag, "CC") == 0 ||
		strcmp(tag, "cc") == 0) {
	iscoord[idx] = true;
      }
    }

    is.close();

    /* copy the vectors into the arrays */
    assert(ispunc.size() == isverb.size());
    assert(iscoord.size() == isverb.size());
    _ntags = ispunc.size();
    _tag_is_verb = new bool[_ntags];
    _tag_is_punc = new bool[_ntags];
    _tag_is_coord = new bool[_ntags];
    for(int i = 0; i < _ntags; ++i) {
      _tag_is_verb[i]  = isverb[i];
      _tag_is_punc[i]  = ispunc[i];
      _tag_is_coord[i] = iscoord[i];
    }
  }
}
