/**
 * Implements the token class
 * @file Token.cc
 * @author Mihai Surdeanu
 */

#include "Token.h"
#include "CppAssert.h"
#include "CharUtils.h"

using namespace std;

namespace egstra {

  token::token(const std::string & word,
	       const std::string & lemma,
	       const std::string & coarseTag,
	       const std::string & fineTag,
	       const std::string & feats)
  {
    mWord = toInteger(word);
    RVASSERT(mWord >= 0, "Invalid word index: " << mWord);
    if(lemma != "_"){
      mLemma = toInteger(lemma);
      RVASSERT(mLemma >= 0, "Invalid lemma index: " << mLemma);
    } else {
      mLemma = -1;
    }
    mCoarseTag = toInteger(coarseTag);
    RVASSERT(mCoarseTag >= 0, "Invalid tag index: " << mCoarseTag);
    mFineTag = toInteger(fineTag);
    RVASSERT(mFineTag >= 0, "Invalid tag index: " << mFineTag);
    if(feats != "_"){
      vector<string> tokens;
      simpleTokenize(feats, tokens, " \t|");
      for(size_t i = 0; i < tokens.size(); i ++){
	int feat = toInteger(tokens[i]);
	RVASSERT(feat >= 0, "Invalid morpho feature index: " << feat);
	mMorphoFeats.push_back(feat);
      }
    }
  }
}
