/************************************************************
 *
 *   Author : Terry Koo
 *            maestro@csail.mit.edu
 *
 *   SGDTrainer : trains a weight vector by stochastic gradient
 *                descent in the primal
 *
 ************************************************************/
#ifndef EGSTRA_SGDTRAINER_H
#define EGSTRA_SGDTRAINER_H

#include "Trainer.h"

#include "Example.h"
#include "Parameters.h"

namespace egstra {
  /**
   * SGDTrainer depends on an objective function Obj
   *
   * Obj is expected to provide the following meathods
   *
   *   Obj::exprimal(example* ex, double* S, structured_model M)
   *   Obj::exgrad(parameters g, double scale, example* ex,
   *               double* S, fvec* F, structured_model M)
   *     These return the example-wise contribution to the primal
   *     objective and the primal gradient
   **/
  template<class Obj>
  class sgd_trainer : public trainer {
  private:
    /**** global algorithm parameters *****/
    /* regularization constant */
    double _C;
    /* global initial learning rate (eta0) */
    double _lrate;
    /* primal parameters */
    parameters _W;

    /***** training invocation variables *****/
    /* style of randomization */
    int _rand_type;
    /* seed used for srandom() */
    int _seed;
    /* whether the learning rate is reduced after each example */
    double _anneal;
    /* the decay factor in the adaptive learning rate adjustment */
    double _decay;
    /* squared radius of an L2-ball constraint, or negative if the
       constraint is inactive */
    double _projl2;
    /* whether to write parameter vectors to disk */
    bool _writeparams;
    /* whether to compute the primal objective on each iteration */
    bool _doprimal;

    /* perform a stochastic update on this example */
    void process_example(const example* const ex);

  public:

    sgd_trainer() : trainer() {}
    ~sgd_trainer() {}

    void usage(const char* const mesg = "");
    void process_options();

    void train(dataset& trdata, dataset& valdata, const int T);

  };
}

#include "SGDTrainer.tcc"

#endif /* EGSTRA_SGDTRAINER_H */
